﻿[string] $FilesLocation = "\"
[string] $GAC="GAC"
[string] $ADSS="Active Directory Synchronization Service"
[string] $Azure="Azure"
[string] $ProgramFiles ="Program Files"
[string] $registryPath = "HKLM:\Software\PNMsoft\Sequence\Setup\"
[string] $BRSsrvName="Background Runtime Service"
[string] $JESsrvName="Job Execution Service"
[string] $ADSSsrvName="Active Directory Synchronization Service"
[string] $SharedResources="Shared Resources"




function GetRegistryValue
{
    param
    (
        [Parameter(Mandatory=$true)] [string] $registryPath,
        [Parameter(Mandatory=$true)] [string] $registryKeyName
    )

	$key = Get-Item -LiteralPath $registryPath -ErrorAction SilentlyContinue

    if($key)
    {
	    $key.GetValue($registryKeyName, $null)
    }
	
}

function RegisterDlls 
{
 param
     (
     [Parameter(Mandatory=$true)] [string] $GACFolder,
     [string] $Feature  
    )

    try 
    {
        $assemblies =  Get-ChildItem $GACFolder
        [System.Reflection.Assembly]::Load("System.EnterpriseServices, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")            
        $publish = New-Object System.EnterpriseServices.Internal.Publish 
        foreach ($assembly in $assemblies)
        {
          $assemblyPath = $GACFolder + "\" + $assembly
          $publish.GacInstall($assemblyPath)
        }
        if($Feature)
        {
             Write-Host "$Feature deployment to GAC was successful."
        }
        else
        {
            Write-Host "Deployment to GAC was successful."
        }
    }
    catch [Exception] 
    {
        $ErrorMessage = $_.Exception.Message
        $FailedItem = $_.Exception.ItemName
    }
   
}

function StartService
{
 param 
    (
        [Parameter(Mandatory=$true)] [string] $srvName 
    )

    $maxRepeat = 20
    $status = "Stopped" # change to Stopped if you want to wait for services to start
    try 
    {
        if (Get-Service $srvName -ErrorAction SilentlyContinue)
        {
            start-service $srvName
            Write-Host "Starting" $srvName "..."
            do 
            {
                $count = (Get-Service $srvName | ? {$_.status -eq $status}).count
                $maxRepeat--
                sleep -Milliseconds 600
            } until ($count -eq 0 -or $maxRepeat -eq 0)
        }
    }
    catch [Exception] 
    {
        $ErrorMessage = $_.Exception.Message
        $FailedItem = $_.Exception.ItemName
    }

}


function StopService
{
 param 
    (
        [Parameter(Mandatory=$true)] [string] $srvName 
    )

    $maxRepeat = 20
    $status = "Stopped" # change to Stopped if you want to wait for services to start

    try 
    {  
        stop-service $srvName
        Write-Host "stopping" $srvName "..." 

        do 
        {
            $count = (Get-Service $srvName | ? {$_.status -eq $status}).count
            $maxRepeat--
            sleep -Milliseconds 600
        } until ($count -eq 0 -or $maxRepeat -eq 0)
    }
    catch [Exception] 
    {
        $ErrorMessage = $_.Exception.Message
        $FailedItem = $_.Exception.ItemName
    }

}


function StartServices
{
    if (Get-Service $BRSsrvName -ErrorAction SilentlyContinue)
    {
       StartService -srvName  $BRSsrvName
    }

    if (Get-Service $JESsrvName -ErrorAction SilentlyContinue)
    {
       StartService -srvName  $JESsrvName
    }

    if (Get-Service $ADSSsrvName -ErrorAction SilentlyContinue)
    {
       StartService -srvName  $ADSSsrvName
    }
}


function StopServices
{

    if (Get-Service $BRSsrvName -ErrorAction SilentlyContinue)
    {
       StopService -srvName  $BRSsrvName
    }

    if (Get-Service $JESsrvName -ErrorAction SilentlyContinue)
    {
       StopService -srvName  $JESsrvName
    }

    if (Get-Service $ADSSsrvName -ErrorAction SilentlyContinue)
    {
       StopService -srvName  $ADSSsrvName
    }
}


function CopySharedResources
{
   param 
    (
        [Parameter(Mandatory=$true)] [string] $SharedResourcesPatchPath 
    )    [string] $SharedResourcesLoaclFolder = GetRegistryValue -registryPath $registryPath -registryKeyName "SHARED_RESOURCES"


    if($SharedResourcesLoaclFolder)
    {
        $item= Get-ChildItem $SharedResourcesPatchPath 
        Copy-Item $item.FullName $SharedResourcesLoaclFolder -Recurse  -force
        Write-Host "Shared Resources files successfully deployed."
    }
}


function CopyAzureDll
{
    param 
    (
        [Parameter(Mandatory=$true)] [string] $AzurePatchPath 
    )


    $item = Get-ChildItem $AzurePatchPath     

    if(Test-Path "C:\WINDOWS\Microsoft.NET\assembly\GAC_MSIL\PNMsoft.Sequence.AzureServices") 
    {
       RegisterDlls -GACFolder $item.FullName -Feature "Azure"
    }

    [string] $BRSPath = GetRegistryValue -registryPath $registryPath -registryKeyName "BRSINSTALLDIR" 

    if($BRSPath)
    {
        if(Test-Path $BRSPath"\PNMsoft.Sequence.AzureServices.dll")
        {
             Copy-Item $item.FullName $BRSPath  -force
             Write-Host "AzureServices.dll successfully deployed." 
        }
    }

    if($AdministrationLoaclFolder)
    {
        if(Test-Path $AdministrationLoaclFolder"\bin\PNMsoft.Sequence.AzureServices.dll")
        {
             Copy-Item $item.FullName $AdministrationLoaclFolder"\bin"  -force
             Write-Host "AzureServices.dll to administration successfully deployed." 
        }
    }


    [string] $FlowtimeLoaclFolder = GetRegistryValue -registryPath $registryPath -registryKeyName "FLOWTIMESITEINSTALLDIR" 
       
    if($FlowtimeLoaclFolder)
    {
        if(Test-Path $FlowtimeLoaclFolder"\bin\PNMsoft.Sequence.AzureServices.dll")
        {
             Copy-Item $item.FullName $FlowtimeLoaclFolder"\bin"  -force
             Write-Host "AzureServices.dll to flowtime successfully deployed." 
        }
    }

    [string] $PTGLoaclFolder = GetRegistryValue -registryPath $registryPath -registryKeyName "PROCESSTOGOSITEINSTALLDIR"

    if($PTGLoaclFolder)
    {
        if(Test-Path $PTGLoaclFolder"\bin\PNMsoft.Sequence.AzureServices.dll")
        {
             Copy-Item $item.FullName $PTGLoaclFolder"\bin"  -force
             Write-Host "AzureServices.dll to PTG successfully deployed." 
        }
    }       
}


[string] $AdministrationLoaclFolder = GetRegistryValue -registryPath $registryPath -registryKeyName "ADMINISTRATIONSITEINSTALLDIR" 

$dir=Get-ChildItem 

StopServices
foreach ($d in $dir)
{
   switch ($d.Name) 
    { 
       $GAC  {RegisterDlls -GACFolder $d.FullName}    
       $SharedResources  {CopySharedResources -SharedResourcesPatchPath  $d.FullName}  
       $Azure {CopyAzureDll -AzurePatchPath  $d.FullName}
    }
}


Write-Host "Executing iisreset..."
iisreset
StartServices


